import React, { useEffect, useState } from "react";
import { useForm } from "./useForm";
import { TextField } from "@mui/material";
import Select from "./controls/Select";
import Input from "./controls/Input";
import Button from "@mui/material/Button";
import AdapterDateFns from "@mui/lab/AdapterDateFns";
import LocalizationProvider from "@mui/lab/LocalizationProvider";
import Box from "@mui/material/Box";
import DateTimePicker from "@mui/lab/DateTimePicker";
import Grid from "@mui/material/Grid";
import { useHistory } from "react-router-dom";
import { fetchTypes } from "../helpers/types";
import { fetchSigns } from "../helpers/signs";
import { postEmail } from "../helpers/emails";
import { makeStyles } from "@mui/styles";

// style for form wrapper
const useStyles = makeStyles(() => ({
  root: {
    "& .MuiFormControl-root": {
      width: "80%",
      margin: 10,
    },
  },
}));

// EmailForm wrapper
export function Form(props) {
  const classes = useStyles();
  const { children, ...other } = props;
  return (
    <form className={classes.root} autoComplete="off" {...other}>
      {props.children}
    </form>
  );
}

// initial email values
const initialFValues = {
  subject: "",
  sender: "",
  recipient: "",
  date: new Date(),
  cc: "",
  body: "",
  type: 1,
  sign: [],
};

// Form is used for adding new emails
export default function EmailForm() {
  // hook for all types
  const [types, setTypes] = useState([]);
  // hook for all signs
  const [signs, setSigns] = useState([]);
  const history = useHistory();

  // on page mount load all types and signs
  useEffect(() => {
    fetchTypes().then((res) => setTypes(res));
    fetchSigns().then((res) => setSigns(res));
  }, []);

  // validation on inserted data
  const validate = (fieldValues = values) => {
    let temp = { ...errors };

    let type = fieldValues.type;

    // subject field cannot be empty
    temp.subject = fieldValues.subject ? "" : "Subject is required.";
    // subject field cannot be empty
    temp.cc = fieldValues.cc ? "" : "Cc is required.";
    // subject field cannot be empty
    temp.recipient = fieldValues.recipient ? "" : "Recipient is required.";
    // sender field cannot be empty
    temp.sender = fieldValues.sender ? "" : "Sender is required.";
    // body field cannot be empty
    temp.body = fieldValues.body ? "" : "Body is required.";
    // type field cannot be empty
    temp.type = fieldValues.type ? "" : "Type is required.";

    if (type === 1 && fieldValues.sign.length !== 0) {
      // legitimate email cannot have signs
      temp.sign = "Legitimate email cannot have signs";
    } else if (type !== 1) {
      // phishing email should have at least one sigh
      temp.sign =
        fieldValues.sign.length !== 0 ? "" : "Choose at least one sign.";
    } else {
      temp.sign = "";
    }

    if (!Date.parse(fieldValues.date)) {
      // if date is in bad format set error
      temp.date = "Date time in bad format";
    } else {
      temp.date = "";
    }

    setErrors({
      ...temp,
    });

    // return true if no errors has been set, otherwise false
    return Object.values(temp).every((x) => x === "");
  };

  const {
    values = initialFValues,
    setValues,
    errors,
    setErrors,
    handleInputChange,
  } = useForm(initialFValues, validate); // function handling values changing and changing errors

  // handler for submitting email
  const handleSubmit = (e) => {
    e.preventDefault();

    if (validate()) {
      // if input values are valid, create email and go to previous page
      postEmail(values).then(history.goBack());
    }
  };

  return (
    <Box>
      <Form onSubmit={handleSubmit} style={{ marginTop: 30 }}>
        <Grid container>
          <Grid item container justifyContent="center" xs={12} md={6}>
            {/* Input field for subject of email */}
            <Input
              name="subject"
              label="Subject"
              value={values.subject}
              onChange={handleInputChange}
              error={errors.subject}
            />
            {/* Input field for sender of email */}
            <Input
              label="Sender"
              name="sender"
              value={values.sender}
              onChange={handleInputChange}
              error={errors.sender}
            />
            {/* Input field for recipient of email */}
            <Input
              label="Recipient"
              name="recipient"
              value={values.recipient}
              onChange={handleInputChange}
              error={errors.recipient}
            />
            <LocalizationProvider dateAdapter={AdapterDateFns}>
              <DateTimePicker
                ampm={false}
                label="DateTime picker"
                validationError
                inputFormat="yyyy/MM/dd HH:mm:SS"
                value={values.date}
                onChange={(value) => setValues({ ...values, date: value })}
                renderInput={(params) => <TextField {...params} />}
              />
            </LocalizationProvider>
          </Grid>

          <Grid item container justifyContent="center" xs={12} md={6}>
            {/* Input field for Cc of email */}
            <Input
              label="Cc"
              name="cc"
              value={values.cc}
              onChange={handleInputChange}
              error={errors.cc}
            />
            {/* Input field for body of email */}
            <Input
              label="Body"
              name="body"
              value={values.body}
              multiline={true}
              onChange={handleInputChange}
              error={errors.body}
            />
            {/* Select field for selecting type of email */}
            <Select
              name="type"
              label="Type"
              value={values.type}
              onChange={handleInputChange}
              options={types}
              error={errors.type}
              type={"type"}
            />
            {/* Select field for multi selecting signs of emails */}
            <Select
              name="sign"
              label="Signs"
              value={values.sign}
              onChange={handleInputChange}
              options={signs}
              multiple={true}
              type={"sign"}
              error={errors.sign}
            />
          </Grid>
          <Grid item container xs={12} justifyContent="center">
            {/* Submit button */}
            <Button
              type="submit"
              size="large"
              variant="contained"
              sx={{ mt: 5 }}
            >
              Create
            </Button>
          </Grid>
        </Grid>
      </Form>
    </Box>
  );
}
